%% Matlab file to calculate and plot the
% anisotropic Drucker-Prager failure surface
% written by Gertraud Medicus and Mahdi Taiebat within the project
% 'Animating Soil Models' - 02/2022
% University of Innsbruck, Austria
% University of British Columbia, Canada
% -------------------------------------------------------------------
% useful information is in e.g. Griffiths (1990):
% 'Failure Criteria Interpretation Based on Mohr Coulomb Friction'
% https://doi.org/10.1061/(ASCE)0733-9410(1990)116:6(986)
% and Griffiths & Huang (2009):
% https://doi.org/10.1002/nag.810
% -------------------------------------------------------------------

close all
clear all
clc

disp('================================================')
disp('                                                ')
disp('  anisotropic Drucker-Prager failure surface    ')
disp('              for phi~=0 and c=0                ')
disp('            including anisotropy                ')
disp('   Gertraud Medicus, Mahdi Taiebat (02/2022)    ')
disp('     https://soilmodels.com/soilanim/           ')
disp('                                                ')
disp('================================================')

%% choose friction angle
phi = 15*pi/180; % in radian
M=6*sin(phi)/(3-sin(phi));
%% define axis for anisotropy a in principal stress space, see Figure(1)
a=[1.5 1 .7];
a=a/(sum(a)/3);

%% convert a in 'sigma-space' to alpha in 'stress-ratio pi-plane' for Figure(2)
p_plot=100; %% chose mean effective stress
alpha=(a*p_plot-p_plot)./p_plot; % 

%% f=0 according to Drucker-Prager
Theta = -180:180;
j=1;
for p=[1 p_plot] % mean effective stress
    for i = Theta
        theta = i*pi/180;

        % Drucker-Prager, see e.g. Griffiths (1990) eq. 14      
        q=M*p;
        %t= sqrt(2/3)*q; 
                
        theta = i*pi/180;
        %% e.g. Griffiths & Huang (2009) eq. (11) and here extended by anisotropy (=cone rotated around the a axis)        
        sigma(j,1) = (p*a(1)+2/3*q*sin(theta-2*pi/3));
        sigma(j,2) = (p*a(2)+2/3*q*sin(theta));
        sigma(j,3) = (p*a(3)+2/3*q*sin(theta+2*pi/3));
        
        j=j+1;
    end
    
end

%% reshape coordinates for 3D plot
N=2;
siz = size(sigma(:,1)',2)/N;
x = reshape(sigma(:,2)',siz,N);
y = reshape(sigma(:,3)',siz,N);
z = reshape(sigma(:,1)',siz,N);
%% 3D plot
figure(1)
hold on;set(gcf, 'units','normalized','outerposition',[0.5 0.5 .15 .5]);
set(gcf,'defaultaxesfontsize',10);
set(gcf,'defaulttextfontsize',10);
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [8.3 8.3]);
set(gcf, 'PaperPosition', [0 0 8.3 8.3]);
cone_clr=[243/255 146/255 0]; % color cone
line_clr=cone_clr*.5;  % color line
iso_clr=[0 0.2 .4]; % color iso ax
colormap(cone_clr) % colormap

%% 3D surface, facea to control transparency
colormap(cone_clr) % colormap cone
surf(x,y,z,'Facea',1); hold on
plot3(x(end-360:end),y(end-360:end),z(end-360:end),'-','Color',line_clr); hold on
%% hydrostatic axis
iso=[0,0,0;350,350,350]';
plot3(iso(1,:),iso(2,:),iso(3,:),'-','Color',iso_clr,'Linewidth',.5); hold on
%% a axis
ax=[0,0,0;a(1),a(2),a(3)]';
plot3(ax(2,:)*p,ax(3,:)*p,ax(1,:)*p,'-','Color',line_clr,'Linewidth',.5); hold on
plot3(a(2)*p_plot,a(3)*p_plot,a(1)*p_plot,'.','Color',line_clr,'Markersize',20); hold on

%% axes
plot3([-10 3*p_plot],[0 0],[0 0],'-','Color','k','Linewidth',.5);
plot3([0 0],[-10 3*p_plot],[0 0],'-','Color','k','Linewidth',.5);
plot3([0 0],[0 0],[-10 3*p_plot],'-','Color','k','Linewidth',.5);
text(0,0,310,'$$\sigma_1''$$', 'Interpreter','Latex');hold on
text(310,0,0,'$$\sigma_2''$$', 'Interpreter','Latex');hold on
text(0,310,0,'$$\sigma_3''$$', 'Interpreter','Latex');hold on

%% pi plane for p=p_plot
P=[3*p_plot 0 0];
plot3([p_plot P(1)],[p_plot P(2)],[p_plot P(3)],':','Color','k','Linewidth',.5);
plot3([p_plot P(2)],[p_plot P(1)],[p_plot P(3)],':','Color','k','Linewidth',.5);
plot3([p_plot P(2)],[p_plot P(3)],[p_plot P(1)],':','Color','k','Linewidth',.5);

axis equal; grid off; axis off;
set(gca,'LooseInset',get(gca,'TightInset')); % remove border around figure
%
xlim([-inf inf]);
ylim([-inf inf]);
zlim([-inf inf]);

view([80 15]);
%view([135 atand(1/sqrt(2))]); % view deviatoric plane
%% light settings
l1 = light;
l1.Position = [.5 -1 .4];
l1.Color = [1 1 1];

s.FaceLighting = 'phong';
s.AmbientStrength = 1;
s.DiffuseStrength = 1; 
s.BackFaceLighting = 'lit';

s.SpecularStrength = .5;
s.SpecularColorReflectance = 0;
s.SpecularExponent = 25;

camlight('left')
shading interp

%% 
figure(2)
hold on;set(gcf, 'units','normalized','outerposition',[.65 .5 .15 .5]);
set(gcf,'defaultaxesfontsize',10);
set(gcf,'defaulttextfontsize',10);
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [8.3 8.3]);
set(gcf, 'PaperPosition', [0 0 8.3 8.3]);

%% DP aniso:
plot3((x(end-360:end)-p_plot)./p_plot,(y(end-360:end)-p_plot)./p_plot,(z(end-360:end)-p_plot)./p_plot,'-','Color',line_clr); hold on

%% alpha axis
plot3(alpha(2),alpha(3),alpha(1),'.','Color',line_clr,'Markersize',20); hold on

%% axes in stress-ratio pi plane = point in stress-ration pi-plane, through the origin
P_0=P/(sum(P)/3);
P_oct=(P_0*p_plot-p_plot)./p_plot; 
plot3([0 P_oct(1)],[0 P_oct(2)],[0 P_oct(3)],'-','Color','k','Linewidth',1);
plot3([0 P_oct(2)],[0 P_oct(1)],[0 P_oct(3)],'-','Color','k','Linewidth',1);
plot3([0 P_oct(2)],[0 P_oct(3)],[0 P_oct(1)],'-','Color','k','Linewidth',1);
text(P_oct(2)*1.1,P_oct(3)*1.1,P_oct(1)*1.1,'$$r_1$$', 'Interpreter','Latex','Color','k');hold on
text(P_oct(1)*1.1,P_oct(2)*1.1,P_oct(3)*1.1,'$$r_2$$', 'Interpreter','Latex','Color','k');hold on
text(P_oct(2)*1.1,P_oct(1)*1.1,P_oct(3)*1.1,'$$r_3$$', 'Interpreter','Latex','Color','k');hold on

%% plane
P_oct2=[P_oct(2)  P_oct(3) P_oct(1)];
P_oct3=[P_oct(2)  P_oct(1) P_oct(3)];
points=[P_oct' P_oct2' P_oct3']; % 
plane=fill3(points(1,:),points(2,:),points(3,:),'c');
set(plane,'facealpha',.075, 'Edgecolor','c');
%% axis in stress space
plot3([-.1 3],[0 0],[0 0],':','Color','k','Linewidth',.5);
plot3([0 0],[-.1 3],[0 0],':','Color','k','Linewidth',.5);
plot3([0 0],[0 0],[-.1 3],':','Color','k','Linewidth',.5);

axis equal; grid off; axis off;
set(gca,'LooseInset',get(gca,'TightInset')); % remove border around figure
%
xlim([-inf inf]);
ylim([-inf inf]);
zlim([-inf inf]);

%%view([80 15]);
view([135 atand(1/sqrt(2))]); % view deviatoric plane


%% print
% print figure
figure(1)
print(gcf,'-r300','-dpng','DP_aniso_3D.png')
figure(2)
print(gcf,'-r300','-dpng','DP_aniso.png')