%% MATLAB script to calculate and plot the
% SANISAND surfaces
% by Gertraud Medicus, Mahdi Taiebat within the project
% 'Animating Soil Models' - 02/2022
% University of Innsbruck, Austria
% University of British Columbia, Canada
% -------------------------------------------------------------------
% further information is in
% Yang, Taiebat & Dafalias (2022) 'SANISAND-MSf: a sand plasticity model with memory surface and
% semifluidised state', Géotechnique, https://doi.org/10.1680/jgeot.19.P.363
% and
% Griffiths (1990) 'Failure Criteria Interpretation Based on Mohr Coulomb Friction', 
% Journal of Geotechnical Engineering, https://doi.org/10.1061/(ASCE)0733-9410(1990)116:6(986)
% -------------------------------------------------------------------

close all
clear all
clc

disp('================================================')
disp('                                                ')
disp('                   SANISAND                     ')
disp('                   surfaces                     ')
disp('  Gertraud Medicus, Mahdi Taiebat (02/2022)     ')
disp('     https://soilmodels.com/soilanim/           ')
disp('                                                ')
disp('================================================')

%% fill in parameters:
c=0.8;
m=0.05;
% m_M=0.35;
M_d=0.65;
M_c=1.28;
M_b=1.4;

%% define axis for anisotropy a in principal stress space, yield
% Note that x, y, z are 2, 3, 1-axes, see figure(1):
a_y = [.5,.2,.2];  %tensor of axis of yield surface
a_y=a_y/(sum(a_y)/3);
%% convert a in 'sigma-space' to alpha in 'stress-ratio pi-plane'
p_plot=100; %*sqrt(3); %% chose mean effective stress
alpha_y=(a_y*p_plot-p_plot)./p_plot; % 


%% define axis for anisotropy a in principal stress space, memory
a_M = [.3,.2,.2];  %tensor of axis of yield surface
a_M=a_M/(sum(a_M)/3);
%% convert a in 'sigma-space' to alpha in 'stress-ratio pi-plane'
p_plot=100; %*sqrt(3); %% chose mean effective stress
alpha_M=(a_M*p_plot-p_plot)./p_plot; % 

%Consistency condition for Memory surface, i.e. alpha must be inside or on
%the memory surface: set fM =< 0 by replacing alph^M_theta with alpha in
%Eq. 2. The result gives a lower bound for mM.

m_M_LowerBound = sqrt( ...
(alpha_y(1) - alpha_M(1)) * (alpha_y(1) - alpha_M(1))+ ...
(alpha_y(2) - alpha_M(2)) * (alpha_y(2) - alpha_M(2))+ ...
(alpha_y(3) - alpha_M(3)) * (alpha_y(3) - alpha_M(3)) ...
) / sqrt(2/3);

m_M = m_M_LowerBound * 1.0; % This multiplier shoudl be greater or equal to 1

% alpha_y=alpha_y/(sum(alpha_y)/3); %normalized axis
% alpha_M=alpha_M/(sum(alpha_M)/3); %normalized axis

M=[m,m_M,M_d,M_c,M_b]; 

%% Lode angle
Theta = -180:1:180;

for cone=1:5
%% YIELD AND MEMORY SURFACES
if cone==1 % DP yield
    alpha=a_y';
elseif cone==2 % DP memory
    alpha=a_M';
end    

if cone<3
    sigma=[];
    j=1;
for p=[1 100] % mean effective stress
    for i = Theta
       
        theta = i*pi/180;
        q=M(cone)*p;

        %% DRUCKER PRAGER e.g. Griffiths & Huang (2009) eq. (11) and here extended by anisotropy (=cone rotated around the alpha axis)        
        sigma(j,1) = (p*alpha(1)+2/3*q*sin(theta-2*pi/3));
        sigma(j,2) = (p*alpha(2)+2/3*q*sin(theta));
        sigma(j,3) = (p*alpha(3)+2/3*q*sin(theta+2*pi/3));
        
        j=j+1;
    end
    
end
end
%% SANISAND surfaces: critical, bounding, dilatancy 
if cone>2
       sigma=[]; 
    j=1;
for p=[1 100] % mean effective stress
    for i = Theta
        theta = i*pi/180; 

alpha_c=M(cone)*2*c/((1+c)-(1-c)*cos(3*theta+90*pi/180));
q= alpha_c*p; %

        theta = i*pi/180;
        %% e.g. Griffiths & Huang (2009) eq. (11) and here extended by anisotropy (=cone rotated around the alpha axis)        
        sigma(j,1) = (p+2/3*q*sin(theta-2*pi/3));
        sigma(j,2) = (p+2/3*q*sin(theta));
        sigma(j,3) = (p+2/3*q*sin(theta+2*pi/3));
        
        j=j+1;
    end
    
end
end
%% reshape coordinates for 3D plot
N=2;
siz = size(sigma(:,1)',2)/N;
x = reshape(sigma(:,2)',siz,N);
y = reshape(sigma(:,3)',siz,N);
z = reshape(sigma(:,1)',siz,N);
%% plot surfaces in different colors:
zmin = min(z(:));
zmax = max(z(:));
cdh = min(1,round(0*(z-zmin)/(zmax-zmin))+1);


%% 3D plot
figure(1)
set(gcf,'defaultaxesfontsize',10);
set(gcf,'defaulttextfontsize',10);
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [4.5 4.5]);
set(gcf, 'PaperPosition', [0 0 4.5 4.5]);

% define colors for the cones
cone_clr=[.8 .8 .8;
  243/255 146/255 0; 
    0.0 0.6 0.0;
    1 0 0;
    0  0 .7]; 
colormap(cone_clr) % colormap
iso_clr=[0 0.2 .4]; % color hydrostatic axis

%% 3D surface, facealpha to control transparency
h(cone)=surf(x,y,z,'FaceAlpha',.5); hold on%
set(h(cone),'CData',cdh+cone-1)
plot3(x(end-2*360:end),y(end-2*360:end),z(end-2*360:end),'-','Color',cone_clr(cone,:),'Linewidth',1); hold on


%% hydrostatic axis
iso=[0,0,0;200,200,200]';
plot3(iso(1,:),iso(2,:),iso(3,:),'--','Color',iso_clr,'Linewidth',.5); hold on

%% alpha axis
if cone <3
alphax=[0,0,0;alpha(2),alpha(3),alpha(1)]';
ax(cone)=plot3(alphax(1,:)*p,alphax(2,:)*p,alphax(3,:)*p,'--','Color',cone_clr(cone,:),'Linewidth',.5); hold on
end

%% axes
plot3([-10 200],[0 0],[0 0],'-','Color','k','Linewidth',.5);
plot3([0 0],[-10 200],[0 0],'-','Color','k','Linewidth',.5);
plot3([0 0],[0 0],[-10 200],'-','Color','k','Linewidth',.5);
text(0,-5,210,'$$\sigma_1''$$', 'Interpreter','Latex');hold on
text(210,0,0,'$$\sigma_2''$$', 'Interpreter','Latex');hold on
text(0,210,0,'$$\sigma_3''$$', 'Interpreter','Latex');hold on

axis equal; grid off; axis off;
set(gca,'LooseInset',get(gca,'TightInset')); % remove border around figure
%
xlim([-inf inf]);
ylim([-inf inf]);
zlim([-inf inf]);

view([80 15]);
%view([135 atand(1/sqrt(2))]); % view deviatoric plane

%% light settings
l1 = light;
l1.Position = [.5 -1 .4];
l1.Color = [1 1 1];

s.FaceLighting = 'phong';
s.AmbientStrength = 1;
s.DiffuseStrength = 1; 
s.BackFaceLighting = 'lit';

s.SpecularStrength = .5;
s.SpecularColorReflectance = 0;
s.SpecularExponent = 25;

%camlight('left')
shading interp
end


%% print figure
figure(1)
sanisand_3D = ['sanisand_3D.png'];
print(gcf,'-r300','-dpng',sanisand_3D)



